# SPDX-FileCopyrightText: 2022 flom84 for Adafruit Industries
# SPDX-License-Identifier: MIT

import time
import board
import displayio
import terminalio

from adafruit_button import Button
from adafruit_cursorcontrol.cursorcontrol import Cursor
from adafruit_cursorcontrol.cursorcontrol_cursormanager import DebouncedCursorManager

SELECT_BUTTON_X = 0
SELECT_BUTTON_Y = 103
SELECT_BUTTON_WIDTH = 60
SELECT_BUTTON_HEIGHT = 25
SELECT_BUTTON_STYLE = Button.ROUNDRECT
SELECT_BUTTON_OUTLINE_COLOR = 0xFFFFFF
SELECT_BUTTON_LABEL = "Select"
SELECT_BUTTON_LABEL_COLOR = 0xFFFFFF

START_BUTTON_X = 100
START_BUTTON_Y = 103
START_BUTTON_WIDTH = 60
START_BUTTON_HEIGHT = 25
START_BUTTON_STYLE = Button.ROUNDRECT
START_BUTTON_OUTLINE_COLOR = 0xFFFFFF
START_BUTTON_LABEL = "Start"
START_BUTTON_LABEL_COLOR = 0xFFFFFF

A_BUTTON_X = 120
A_BUTTON_Y = 20
A_BUTTON_WIDTH = 30
A_BUTTON_HEIGHT = 30
A_BUTTON_STYLE = Button.ROUNDRECT
A_BUTTON_OUTLINE_COLOR = 0xFFFFFF
A_BUTTON_LABEL = "A"
A_BUTTON_LABEL_COLOR = 0xFFFFFF

B_BUTTON_X = 80
B_BUTTON_Y = 30
B_BUTTON_WIDTH = 30
B_BUTTON_HEIGHT = 30
B_BUTTON_STYLE = Button.ROUNDRECT
B_BUTTON_OUTLINE_COLOR = 0xFFFFFF
B_BUTTON_LABEL = "B"
B_BUTTON_LABEL_COLOR = 0xFFFFFF

BUTTON_FILL_PURPLE = 0xB400FF
BUTTON_FILL_BLACK = 0x000000

start_button = Button(
    x=START_BUTTON_X,
    y=START_BUTTON_Y,
    width=START_BUTTON_WIDTH,
    height=START_BUTTON_HEIGHT,
    style=START_BUTTON_STYLE,
    fill_color=BUTTON_FILL_BLACK,
    outline_color=START_BUTTON_OUTLINE_COLOR,
    label=START_BUTTON_LABEL,
    label_font=terminalio.FONT,
    label_color=START_BUTTON_LABEL_COLOR,
)

select_button = Button(
    x=SELECT_BUTTON_X,
    y=SELECT_BUTTON_Y,
    width=SELECT_BUTTON_WIDTH,
    height=SELECT_BUTTON_HEIGHT,
    style=SELECT_BUTTON_STYLE,
    fill_color=BUTTON_FILL_BLACK,
    outline_color=SELECT_BUTTON_OUTLINE_COLOR,
    label=SELECT_BUTTON_LABEL,
    label_font=terminalio.FONT,
    label_color=SELECT_BUTTON_LABEL_COLOR,
)

a_button = Button(
    x=A_BUTTON_X,
    y=A_BUTTON_Y,
    width=A_BUTTON_WIDTH,
    height=A_BUTTON_HEIGHT,
    style=A_BUTTON_STYLE,
    fill_color=BUTTON_FILL_BLACK,
    outline_color=A_BUTTON_OUTLINE_COLOR,
    label=A_BUTTON_LABEL,
    label_font=terminalio.FONT,
    label_color=A_BUTTON_LABEL_COLOR,
)

b_button = Button(
    x=B_BUTTON_X,
    y=B_BUTTON_Y,
    width=B_BUTTON_WIDTH,
    height=B_BUTTON_HEIGHT,
    style=B_BUTTON_STYLE,
    fill_color=BUTTON_FILL_BLACK,
    outline_color=B_BUTTON_OUTLINE_COLOR,
    label=B_BUTTON_LABEL,
    label_font=terminalio.FONT,
    label_color=B_BUTTON_LABEL_COLOR,
)

# Create the display
display = board.DISPLAY

# Create the display context
splash = displayio.Group()

# initialize the mouse cursor object
mouse_cursor = Cursor(display, display_group=splash)

# initialize the debounced cursor manager
debounced_cursor = DebouncedCursorManager(mouse_cursor)

# create displayio group
splash.append(start_button)
splash.append(select_button)
splash.append(a_button)
splash.append(b_button)
display.show(splash)

while True:
    debounced_cursor.update()

    if debounced_cursor.is_clicked:
        a_button.fill_color = BUTTON_FILL_PURPLE
        print("A pressed:  " + str(debounced_cursor.held))

    if debounced_cursor.released:
        a_button.fill_color = BUTTON_FILL_BLACK
        print("A pressed:  " + str(debounced_cursor.held))

    if debounced_cursor.is_alt_clicked:
        b_button.fill_color = BUTTON_FILL_PURPLE
        print("B pressed:  " + str(debounced_cursor.alt_held))

    if debounced_cursor.alt_released:
        b_button.fill_color = BUTTON_FILL_BLACK
        print("B pressed:  " + str(debounced_cursor.alt_held))

    if debounced_cursor.is_start_clicked:
        start_button.fill_color = BUTTON_FILL_PURPLE
        print("Start pressed: " + str(debounced_cursor.start_held))

    if debounced_cursor.start_released:
        start_button.fill_color = BUTTON_FILL_BLACK
        print("Start pressed: " + str(debounced_cursor.start_held))

    if debounced_cursor.is_select_clicked:
        select_button.fill_color = BUTTON_FILL_PURPLE
        print("Select pressed:  " + str(debounced_cursor.select_held))

    if debounced_cursor.select_released:
        select_button.fill_color = BUTTON_FILL_BLACK
        print("Select pressed:  " + str(debounced_cursor.select_held))

    time.sleep(0.01)



# SPDX-FileCopyrightText: 2019 Brent Rubell for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
`adafruit_cursorcontrol.cursorcontrol_cursormanager`
================================================================================
Mouse cursor for interaction with CircuitPython UI elements

* Author(s): Brent Rubell

"""
__version__ = "2.7.4"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_CursorControl.git"



# SPDX-FileCopyrightText: 2019 Brent Rubell for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_cursorcontrol.cursorcontrol`
================================================================================

Mouse cursor for interaction with CircuitPython UI elements.


* Author(s): Brent Rubell

Implementation Notes
--------------------

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://github.com/adafruit/circuitpython/releases

"""
import displayio

try:
    from typing import Optional, Type
    from types import TracebackType
except ImportError:
    pass

__version__ = "2.7.4"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_CursorControl.git"


class Cursor:
    """Mouse cursor interaction for CircuitPython.

    :param ~displayio.Display display: CircuitPython display object.
    :param ~displayio.Group display_group: CircuitPython group object to append the cursor to.
    :param ~displayio.Bitmap bmp: CircuitPython bitmap object to use as the cursor
    :param bool is_hidden: Cursor is hidden on init.
    :param int cursor_speed: Speed of the cursor, in pixels.
    :param int scale: Scale amount for the cursor in both directions.

    Example for creating a cursor layer

    .. code-block:: python

        from adafruit_cursorcontrol import Cursor
        # Create the display
        display = board.DISPLAY

        # Create the display context
        splash = displayio.Group()

        # initialize the mouse cursor object
        mouse_cursor = Cursor(display, display_group=splash)
    """

    # pylint: disable=too-many-arguments,line-too-long
    def __init__(
        self,
        display: Optional[displayio.Display] = None,
        display_group: Optional[displayio.Group] = None,
        bmp: Optional[displayio.Bitmap] = None,
        is_hidden: bool = False,
        cursor_speed: int = 5,
        scale: int = 1,
    ):
        self._display = display
        self._scale = scale
        self._speed = cursor_speed
        self._is_hidden = is_hidden
        self._display_grp = display_group
        self._disp_sz = display.height - 1, display.width - 1
        self._cur_sprite = None
        if bmp is None:
            self._cursor_bitmap = self._default_cursor_bitmap()
        else:
            self._cursor_bitmap = bmp
        self.generate_cursor(self._cursor_bitmap)

    # pylint: enable=too-many-arguments,line-too-long

    def __enter__(self) -> "Cursor":
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[type]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None:
        self.deinit()

    def deinit(self) -> None:
        """deinitializes the cursor object."""
        self._is_deinited()
        self._scale = None
        self._display_grp.remove(self._cursor_grp)

    def _is_deinited(self) -> None:
        """checks cursor deinitialization"""
        if self._scale is None:
            raise ValueError(
                "Cursor object has been deinitialized and can no longer "
                "be used. Create a new cursor object."
            )

    @property
    def scale(self) -> int:
        """Returns the cursor's scale amount as an integer."""
        return self._scale

    @scale.setter
    def scale(self, scale_value: int) -> None:
        """Scales the cursor by scale_value in both directions.
        :param int scale_value: Amount to scale the cursor by.
        """
        self._is_deinited()
        if scale_value > 0:
            self._scale = scale_value
            self._cursor_grp.scale = scale_value

    @property
    def speed(self) -> int:
        """Returns the cursor's speed, in pixels."""
        return self._speed

    @speed.setter
    def speed(self, speed: int) -> None:
        """Sets the speed of the cursor.
        :param int speed: Cursor movement speed, in pixels.
        """
        self._is_deinited()
        if speed > 0:
            self._speed = speed

    @property
    def x(self) -> int:
        """Returns the cursor's x-coordinate."""
        return self._cursor_grp.x

    @x.setter
    def x(self, x_val: int) -> None:
        """Sets the x-value of the cursor.
        :param int x_val: cursor x-position, in pixels.
        """
        self._is_deinited()
        if x_val < 0 and not self._is_hidden:
            self._cursor_grp.x = self._cursor_grp.x
        elif x_val > self._disp_sz[1] and not self._is_hidden:
            self._cursor_grp.x = self._cursor_grp.x
        elif not self._is_hidden:
            self._cursor_grp.x = x_val

    @property
    def y(self) -> int:
        """Returns the cursor's y-coordinate."""
        return self._cursor_grp.y

    @y.setter
    def y(self, y_val: int) -> None:
        """Sets the y-value of the cursor.
        :param int y_val: cursor y-position, in pixels.
        """
        self._is_deinited()
        if y_val < 0 and not self._is_hidden:
            self._cursor_grp.y = self._cursor_grp.y
        elif y_val > self._disp_sz[0] and not self._is_hidden:
            self._cursor_grp.y = self._cursor_grp.y
        elif not self._is_hidden:
            self._cursor_grp.y = y_val

    @property
    def hidden(self) -> bool:
        """Returns True if the cursor is hidden or visible on the display."""
        return self._is_hidden

    @hidden.setter
    def hidden(self, is_hidden: bool) -> None:
        self._is_deinited()
        if is_hidden:
            self._is_hidden = True
            self._display_grp.remove(self._cursor_grp)
        else:
            self._is_hidden = False
            self._display_grp.append(self._cursor_grp)

    def hide(self) -> None:
        """Hide the cursor."""
        self.hidden = True

    def show(self) -> None:
        """Show the cursor."""
        self.hidden = False

    # pylint:disable=no-self-use
    def _default_cursor_bitmap(self) -> displayio.Bitmap:
        bmp = displayio.Bitmap(20, 20, 3)
        # left edge, outline
        for i in range(0, bmp.height):
            bmp[0, i] = 2
        # right diag outline, inside fill
        for j in range(1, 15):
            bmp[j, j] = 2
            for i in range(j + 1, bmp.height - j):
                bmp[j, i] = 1
        # bottom diag., outline
        for i in range(1, 5):
            bmp[i, bmp.height - i] = 2
        # bottom flat line, right side fill
        for i in range(5, 15):
            bmp[i, 15] = 2
            bmp[i - 1, 14] = 1
            bmp[i - 2, 13] = 1
            bmp[i - 3, 12] = 1
            bmp[i - 4, 11] = 1
        return bmp

    # pylint:enable=no-self-use

    @property
    def cursor_bitmap(self) -> displayio.Bitmap:
        """Return the cursor bitmap."""
        return self._cursor_bitmap

    @cursor_bitmap.setter
    def cursor_bitmap(self, bmp: displayio.Bitmap) -> None:
        """Set a new cursor bitmap.

        :param ~displayio.Bitmap bmp: A Bitmap to use for the cursor
        """
        self._cursor_bitmap = bmp
        self._cursor_grp.remove(self._cur_sprite)
        self._cur_sprite = displayio.TileGrid(bmp, pixel_shader=self._cur_palette)
        self._cursor_grp.append(self._cur_sprite)

    def generate_cursor(self, bmp: displayio.Bitmap) -> None:
        """Generates a cursor icon

        :param ~displayio.Bitmap bmp: A Bitmap to use for the cursor
        """
        self._is_deinited()
        self._cursor_grp = displayio.Group(scale=self._scale)
        self._cur_palette = displayio.Palette(3)
        self._cur_palette.make_transparent(0)
        self._cur_palette[1] = 0xFFFFFF
        self._cur_palette[2] = 0x0000
        self._cur_sprite = displayio.TileGrid(bmp, pixel_shader=self._cur_palette)
        self._cursor_grp.append(self._cur_sprite)
        self._display_grp.append(self._cursor_grp)



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

import time
import board
import displayio
from adafruit_cursorcontrol.cursorcontrol import Cursor
from adafruit_cursorcontrol.cursorcontrol_cursormanager import CursorManager

# Create the display
display = board.DISPLAY

# Create the display context
splash = displayio.Group()

# initialize the mouse cursor object
bmp = displayio.Bitmap(20, 20, 3)
for i in range(0, bmp.height):
    bmp[0, i] = 1
    bmp[bmp.width - 1, i] = 1
for i in range(0, bmp.width):
    bmp[i, 0] = 1
    bmp[i, bmp.height - 1] = 1

mouse_cursor = Cursor(display, display_group=splash, bmp=bmp)

# initialize the cursormanager
cursor = CursorManager(mouse_cursor)

# show displayio group
display.show(splash)

while True:
    cursor.update()
    if cursor.is_clicked:
        if mouse_cursor.hidden:
            mouse_cursor.show()
        else:
            mouse_cursor.hide()
    time.sleep(0.01)



# SPDX-FileCopyrightText: 2019 Brent Rubell for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_cursorcontrol.cursorcontrol_cursormanager`
================================================================================
Simple interaction user interface interaction for Adafruit_CursorControl.
* Author(s): Brent Rubell
"""
import board
from micropython import const
import analogio
from keypad import ShiftRegisterKeys, Event
from adafruit_debouncer import Debouncer

try:
    from typing import Optional, Type
    from types import TracebackType
    from adafruit_cursorcontrol.cursorcontrol import Cursor
except ImportError:
    pass

__version__ = "2.7.4"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_CursorControl.git"


# PyBadge
PYBADGE_BUTTON_LEFT = const(7)
PYBADGE_BUTTON_UP = const(6)
PYBADGE_BUTTON_DOWN = const(5)
PYBADGE_BUTTON_RIGHT = const(4)
# PyBadge & PyGamer
PYBADGE_BUTTON_SELECT = const(3)
PYBADGE_BUTTON_START = const(2)
PYBADGE_BUTTON_A = const(1)
PYBADGE_BUTTON_B = const(0)


class CursorManager:
    """Simple interaction user interface interaction for Adafruit_CursorControl.

    :param Cursor cursor: The cursor object we are using.
    """

    # pylint: disable=too-many-instance-attributes

    def __init__(self, cursor: Cursor) -> None:
        self._cursor = cursor
        self._is_clicked = False
        self._is_alt_clicked = False
        self._is_select_clicked = False
        self._is_start_clicked = False
        self._pad_states = 0
        self._event = Event()
        self._init_hardware()

    def __enter__(self) -> "CursorManager":
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[type]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None:
        self.deinit()

    def deinit(self) -> None:
        """Deinitializes a CursorManager object."""
        self._is_deinited()
        self._pad.deinit()
        self._cursor.deinit()
        self._cursor = None
        self._event = None

    def _is_deinited(self) -> None:
        """Checks if CursorManager object has been deinitd."""
        if self._cursor is None:
            raise ValueError(
                "CursorManager object has been deinitialized and can no longer "
                "be used. Create a new CursorManager object."
            )

    def _init_hardware(self) -> None:
        """Initializes PyBadge or PyGamer hardware."""
        if hasattr(board, "BUTTON_CLOCK") and not hasattr(board, "JOYSTICK_X"):
            self._pad_btns = {
                "btn_left": PYBADGE_BUTTON_LEFT,
                "btn_right": PYBADGE_BUTTON_RIGHT,
                "btn_up": PYBADGE_BUTTON_UP,
                "btn_down": PYBADGE_BUTTON_DOWN,
                "btn_a": PYBADGE_BUTTON_A,
                "btn_b": PYBADGE_BUTTON_B,
                "btn_select": PYBADGE_BUTTON_SELECT,
                "btn_start": PYBADGE_BUTTON_START,
            }
            self._pad_states = 0
        elif hasattr(board, "JOYSTICK_X"):
            self._joystick_x = analogio.AnalogIn(board.JOYSTICK_X)
            self._joystick_y = analogio.AnalogIn(board.JOYSTICK_Y)
            self._pad_btns = {
                "btn_a": PYBADGE_BUTTON_A,
                "btn_b": PYBADGE_BUTTON_B,
                "btn_select": PYBADGE_BUTTON_SELECT,
                "btn_start": PYBADGE_BUTTON_START,
            }
            # Sample the center points of the joystick
            self._center_x = self._joystick_x.value
            self._center_y = self._joystick_y.value
        else:
            raise AttributeError(
                "Board must have a D-Pad or Joystick for use with CursorManager!"
            )
        self._pad = ShiftRegisterKeys(
            clock=board.BUTTON_CLOCK,
            data=board.BUTTON_OUT,
            latch=board.BUTTON_LATCH,
            key_count=8,
            value_when_pressed=True,
        )

    @property
    def is_clicked(self) -> bool:
        """Returns True if the cursor A button was pressed
        during previous call to update()
        """
        return self._is_clicked

    @property
    def is_alt_clicked(self) -> bool:
        """Returns True if the cursor B button was pressed
        during previous call to update()
        """
        return self._is_alt_clicked

    @property
    def is_select_clicked(self) -> bool:
        """Returns True if the Select button was pressed
        during previous call to update()
        """
        return self._is_select_clicked

    @property
    def is_start_clicked(self) -> bool:
        """Returns True if the Start button was pressed
        during previous call to update()
        """
        return self._is_start_clicked

    def update(self) -> None:
        """Updates the cursor object."""
        if self._pad.events.get_into(self._event):
            self._store_button_states()
        self._check_cursor_movement()
        if self._is_clicked:
            self._is_clicked = False
        elif self._pad_states & (1 << self._pad_btns["btn_a"]):
            self._is_clicked = True

        if self._is_alt_clicked:
            self._is_alt_clicked = False
        elif self._pad_states & (1 << self._pad_btns["btn_b"]):
            self._is_alt_clicked = True

        if self._is_select_clicked:
            self._is_select_clicked = False
        elif self._pad_states & (1 << self._pad_btns["btn_select"]):
            self._is_select_clicked = True

        if self._is_start_clicked:
            self._is_start_clicked = False
        elif self._pad_states & (1 << self._pad_btns["btn_start"]):
            self._is_start_clicked = True

    def _read_joystick_x(self, samples: int = 3) -> float:
        """Read the X analog joystick on the PyGamer.
        :param int samples: How many samples to read and average.
        """
        reading = 0
        # pylint: disable=unused-variable
        if hasattr(board, "JOYSTICK_X"):
            for _ in range(0, samples):
                reading += self._joystick_x.value
            reading /= samples
        return reading

    def _read_joystick_y(self, samples: int = 3) -> float:
        """Read the Y analog joystick on the PyGamer.
        :param int samples: How many samples to read and average.
        """
        reading = 0
        # pylint: disable=unused-variable
        if hasattr(board, "JOYSTICK_Y"):
            for _ in range(0, samples):
                reading += self._joystick_y.value
            reading /= samples
        return reading

    def _store_button_states(self) -> None:
        """Stores the state of the PyBadge's D-Pad or the PyGamer's Joystick
        into a byte
        """
        bit_index = self._event.key_number
        current_state = (self._pad_states >> bit_index) & 1
        if current_state != self._event.pressed:
            self._pad_states = (1 << bit_index) ^ self._pad_states

    def _check_cursor_movement(self) -> None:
        """Checks the PyBadge D-Pad or the PyGamer's Joystick for movement."""
        if hasattr(board, "BUTTON_CLOCK") and not hasattr(board, "JOYSTICK_X"):
            if self._pad_states & (1 << self._pad_btns["btn_right"]):
                self._cursor.x += self._cursor.speed
            elif self._pad_states & (1 << self._pad_btns["btn_left"]):
                self._cursor.x -= self._cursor.speed

            if self._pad_states & (1 << self._pad_btns["btn_up"]):
                self._cursor.y -= self._cursor.speed
            elif self._pad_states & (1 << self._pad_btns["btn_down"]):
                self._cursor.y += self._cursor.speed
        elif hasattr(board, "JOYSTICK_X"):
            joy_x = self._read_joystick_x()
            joy_y = self._read_joystick_y()
            if joy_x > self._center_x + 1000:
                self._cursor.x += self._cursor.speed
            elif joy_x < self._center_x - 1000:
                self._cursor.x -= self._cursor.speed
            if joy_y > self._center_y + 1000:
                self._cursor.y += self._cursor.speed
            elif joy_y < self._center_y - 1000:
                self._cursor.y -= self._cursor.speed
        else:
            raise AttributeError(
                "Board must have a D-Pad or Joystick for use with CursorManager!"
            )


class DebouncedCursorManager(CursorManager):
    """Simple user interface interaction for Adafruit_CursorControl.
    This subclass provide a debounced version on the A, B, Start or Select buttons
    and provides queries for when the buttons are just pressed, and just released,
    as well their current state. "Just" in this context means "since the previous call to update."

    :param Cursor cursor: The cursor object we are using.
    """

    def __init__(self, cursor: Cursor, debounce_interval: float = 0.01) -> None:
        CursorManager.__init__(self, cursor)
        self._debouncers = {}
        for btn in self._pad_btns:
            self._debouncers[btn] = Debouncer(
                lambda btn=btn: bool((self._pad_states & (1 << self._pad_btns[btn]))),
                interval=debounce_interval,
            )

    @property
    def is_clicked(self) -> bool:
        """Returns True if the cursor A button was pressed
        during previous call to update()
        """
        return self._debouncers["btn_a"].rose

    @property
    def is_alt_clicked(self) -> bool:
        """Returns True if the cursor B button was pressed
        during previous call to update()
        """
        return self._debouncers["btn_b"].rose

    @property
    def is_select_clicked(self) -> bool:
        """Returns True if the Select button was pressed
        during previous call to update()
        """
        return self._debouncers["btn_select"].rose

    @property
    def is_start_clicked(self) -> bool:
        """Returns True if the Start button was pressed
        during previous call to update()
        """
        return self._debouncers["btn_start"].rose

    @property
    def released(self) -> bool:
        """Returns True if the cursor A button was released
        during previous call to update()
        """
        return self._debouncers["btn_a"].fell

    @property
    def alt_released(self) -> bool:
        """Returns True if the cursor B button was released
        during previous call to update()
        """
        return self._debouncers["btn_b"].fell

    @property
    def start_released(self) -> bool:
        """Returns True if the cursor Start button was released
        during previous call to update()
        """
        return self._debouncers["btn_start"].fell

    @property
    def select_released(self) -> bool:
        """Returns True if the cursor Select button was released
        during previous call to update()
        """
        return self._debouncers["btn_select"].fell

    @property
    def held(self) -> bool:
        """Returns True if the cursor A button is currently being held"""
        return self._debouncers["btn_a"].value

    @property
    def alt_held(self) -> bool:
        """Returns True if the cursor B button is currently being held"""
        return self._debouncers["btn_b"].value

    @property
    def start_held(self) -> bool:
        """Returns True if the cursor Start button is currently being held"""
        return self._debouncers["btn_start"].value

    @property
    def select_held(self) -> bool:
        """Returns True if the cursor Select is currently being held"""
        return self._debouncers["btn_select"].value

    def update(self) -> None:
        """Updates the cursor object."""
        if self._pad.events.get_into(self._event):
            self._store_button_states()
        self._check_cursor_movement()
        for debouncer in self._debouncers.values():
            debouncer.update()



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

import time
import board
import displayio
from adafruit_button import Button
from adafruit_display_text import label
import terminalio
from adafruit_cursorcontrol.cursorcontrol import Cursor
from adafruit_cursorcontrol.cursorcontrol_cursormanager import CursorManager

# Create the display
display = board.DISPLAY

# Create the display context
splash = displayio.Group()

# Use the built-in system font
font = terminalio.FONT

##########################################################################
# Make a background color fill

color_bitmap = displayio.Bitmap(display.width, display.height, 1)
color_palette = displayio.Palette(1)
color_palette[0] = 0x404040
bg_sprite = displayio.TileGrid(color_bitmap, pixel_shader=color_palette, x=0, y=0)
splash.append(bg_sprite)

##########################################################################

# Set up button/label properties
BUTTON_WIDTH = 80
BUTTON_HEIGHT = 40
BUTTON_MARGIN = 20
LBL_HEADER = [100, 20]
LBL_TEXT = [120, 40]

# Resize buttons for small display (PyGamer)
if display.width < 240:
    BUTTON_WIDTH = int(BUTTON_WIDTH / 2)
    BUTTON_HEIGHT = int(BUTTON_HEIGHT / 2)
    BUTTON_MARGIN = int(BUTTON_MARGIN / 2)
    LBL_HEADER[0] -= 75
    LBL_HEADER[1] -= 10
    LBL_TEXT[0] -= 70
    LBL_TEXT[1] += 55

# Create the buttons
buttons = []

button_speed_inc = Button(
    x=BUTTON_MARGIN,
    y=BUTTON_MARGIN + BUTTON_HEIGHT,
    width=BUTTON_WIDTH,
    height=BUTTON_HEIGHT,
    label="Speed+",
    label_font=font,
)
buttons.append(button_speed_inc)

button_speed_dec = Button(
    x=BUTTON_MARGIN,
    y=BUTTON_MARGIN * 4 + BUTTON_HEIGHT,
    width=BUTTON_WIDTH,
    height=BUTTON_HEIGHT,
    label="Speed-",
    label_font=font,
)
buttons.append(button_speed_dec)

button_scale_pos = Button(
    x=BUTTON_MARGIN * 3 + 2 * BUTTON_WIDTH,
    y=BUTTON_MARGIN + BUTTON_HEIGHT,
    width=BUTTON_WIDTH,
    height=BUTTON_HEIGHT,
    label="Scale+",
    label_font=font,
    style=Button.SHADOWRECT,
)
buttons.append(button_scale_pos)

button_scale_neg = Button(
    x=BUTTON_MARGIN * 3 + 2 * BUTTON_WIDTH,
    y=BUTTON_MARGIN * 4 + BUTTON_HEIGHT,
    width=BUTTON_WIDTH,
    height=BUTTON_HEIGHT,
    label="Scale-",
    label_font=font,
    style=Button.SHADOWRECT,
)
buttons.append(button_scale_neg)

# Show the button
for b in buttons:
    splash.append(b.group)

# Create a text label
text_label = label.Label(
    font, text="CircuitPython Cursor!", color=0x00FF00, x=LBL_HEADER[0], y=LBL_HEADER[1]
)
splash.append(text_label)

text_speed = label.Label(font, color=0x00FF00, x=LBL_TEXT[0], y=LBL_TEXT[1])
splash.append(text_speed)

text_scale = label.Label(font, color=0x00FF00, x=LBL_TEXT[0], y=LBL_TEXT[1] + 20)
splash.append(text_scale)

# initialize the mouse cursor object
mouse_cursor = Cursor(display, display_group=splash)

# initialize the cursormanager
cursor = CursorManager(mouse_cursor)

# show displayio group
display.show(splash)

prev_btn = None
while True:
    cursor.update()
    if cursor.is_clicked is True:
        for i, b in enumerate(buttons):
            if b.contains((mouse_cursor.x, mouse_cursor.y)):
                b.selected = True
                print("Button %d pressed" % i)
                if i == 0:  # Increase the cursor speed
                    mouse_cursor.speed += 1
                elif i == 1:  # Decrease the cursor speed
                    mouse_cursor.speed -= 1
                if i == 2:  # Increase the cursor scale
                    mouse_cursor.scale += 1
                elif i == 3:  # Decrease the cursor scale
                    mouse_cursor.scale -= 1
                prev_btn = b
    elif prev_btn is not None:
        prev_btn.selected = False
    text_speed.text = "Speed: {0}px".format(mouse_cursor.speed)
    text_scale.text = "Scale: {0}px".format(mouse_cursor.scale)
    time.sleep(0.1)
